@@ -1218,3 +1218,27 @@ egress_key: Following value
         with open(yaml_file, 'r') as fhnd:
             filedat = fhnd.read()
         assert filedat == yamlout
+
+    def test_change_null(self, script_runner, tmp_path_factory):
+        yamlin = """---
+ingress_key: Preceding value
+concrete_key:
+egress_key: Following value
+"""
+        yamlout = """---
+ingress_key: Preceding value
+concrete_key: Now not null
+egress_key: Following value
+"""
+        yaml_file = create_temp_yaml_file(tmp_path_factory, yamlin)
+        result = script_runner.run(
+            self.command,
+            "--change=concrete_key",
+            "--value=Now not null",
+            yaml_file
+        )
+        assert result.success, result.stderr
+
+        with open(yaml_file, 'r') as fhnd:
+            filedat = fhnd.read()
+        assert filedat == yamlout
