@@ -6,7 +6,8 @@ Copyright 2020 William W. Kimball, Jr. MBA MSIS
 from itertools import zip_longest
 from typing import Any, Dict, Generator, List, Optional, Tuple
 
-from yamlpath.func import escape_path_section
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
+
 from yamlpath import YAMLPath
 from yamlpath.wrappers import ConsolePrinter, NodeCoords
 from yamlpath.eyaml import EYAMLProcessor
@@ -59,16 +60,17 @@ class Differ:
 
     def _purge_document(self, path: YAMLPath, data: Any):
         """Delete every node in the document."""
-        if isinstance(data, dict):
+        if isinstance(data, CommentedMap):
             lhs_iteration = -1
             for key, val in data.items():
                 lhs_iteration += 1
-                next_path = path + escape_path_section(key, path.seperator)
+                next_path = (path +
+                    YAMLPath.escape_path_section(key, path.seperator))
                 self._diffs.append(
                     DiffEntry(
                         DiffActions.DELETE, next_path, val, None,
                         lhs_parent=data, lhs_iteration=lhs_iteration))
-        elif isinstance(data, list):
+        elif isinstance(data, CommentedSeq):
             for idx, ele in enumerate(data):
                 next_path = path + "[{}]".format(idx)
                 self._diffs.append(
@@ -83,16 +85,17 @@ class Differ:
 
     def _add_everything(self, path: YAMLPath, data: Any) -> None:
         """Add every node in the document."""
-        if isinstance(data, dict):
+        if isinstance(data, CommentedMap):
             rhs_iteration = -1
             for key, val in data.items():
                 rhs_iteration += 1
-                next_path = path + escape_path_section(key, path.seperator)
+                next_path = (path +
+                    YAMLPath.escape_path_section(key, path.seperator))
                 self._diffs.append(
                     DiffEntry(
                         DiffActions.ADD, next_path, None, val,
                         rhs_parent=data, rhs_iteration=rhs_iteration))
-        elif isinstance(data, list):
+        elif isinstance(data, CommentedSeq):
             for idx, ele in enumerate(data):
                 next_path = path + "[{}]".format(idx)
                 self._diffs.append(
@@ -139,7 +142,9 @@ class Differ:
                 DiffEntry(DiffActions.CHANGE, path, lhs, rhs, **kwargs)
             )
 
-    def _diff_dicts(self, path: YAMLPath, lhs: dict, rhs: dict) -> None:
+    def _diff_dicts(
+        self, path: YAMLPath, lhs: CommentedMap, rhs: CommentedMap
+    ) -> None:
         """Diff two dicts."""
         self.logger.debug(
             "Comparing LHS:",
@@ -150,6 +155,22 @@ class Differ:
             prefix="Differ::_diff_dicts:  ",
             data=rhs)
 
+        # Check first for a difference in YAML Tag
+        lhs_tag = lhs.tag.value if hasattr(lhs, "tag") else None
+        rhs_tag = rhs.tag.value if hasattr(rhs, "tag") else None
+        if lhs_tag != rhs_tag:
+            self.logger.debug(
+                "Dictionaries have different YAML Tags; {} != {}:".format(
+                    lhs_tag, rhs_tag),
+                prefix="Differ::_diff_dicts:  ")
+            self._diffs.append(
+                DiffEntry(
+                    DiffActions.DELETE, path, lhs, None, key_tag=lhs_tag))
+            self._diffs.append(
+                DiffEntry(
+                    DiffActions.ADD, path, None, rhs, key_tag=rhs_tag))
+            return
+
         lhs_keys = set(lhs)
         rhs_keys = set(rhs)
         lhs_key_indicies = Differ._get_key_indicies(lhs)
@@ -169,7 +190,8 @@ class Differ:
             (key, val) for key, val in rhs.items()
             if key in lhs and key in rhs
         ]:
-            next_path = path + escape_path_section(key, path.seperator)
+            next_path = (path +
+                YAMLPath.escape_path_section(key, path.seperator))
             self._diff_between(
                 next_path, lhs[key], val,
                 lhs_parent=lhs, lhs_iteration=lhs_key_indicies[key],
@@ -178,23 +200,29 @@ class Differ:
 
         # Look for deleted keys
         for key in lhs_keys - rhs_keys:
-            next_path = path + escape_path_section(key, path.seperator)
+            next_path = (path +
+                YAMLPath.escape_path_section(key, path.seperator))
             self._diffs.append(
                 DiffEntry(
                     DiffActions.DELETE, next_path, lhs[key], None,
                     lhs_parent=lhs, lhs_iteration=lhs_key_indicies[key],
-                    rhs_parent=rhs))
+                    rhs_parent=rhs,
+                    key_tag=key.tag.value if hasattr(key, "tag") else None))
 
         # Look for new keys
         for key in rhs_keys - lhs_keys:
-            next_path = path + escape_path_section(key, path.seperator)
+            next_path = (path +
+                YAMLPath.escape_path_section(key, path.seperator))
             self._diffs.append(
                 DiffEntry(
                     DiffActions.ADD, next_path, None, rhs[key],
                     lhs_parent=lhs,
-                    rhs_parent=rhs, rhs_iteration=rhs_key_indicies[key]))
+                    rhs_parent=rhs, rhs_iteration=rhs_key_indicies[key],
+                    key_tag=key.tag.value if hasattr(key, "tag") else None))
 
-    def _diff_synced_lists(self, path: YAMLPath, lhs: list, rhs: list) -> None:
+    def _diff_synced_lists(
+        self, path: YAMLPath, lhs: CommentedSeq, rhs: CommentedSeq
+    ) -> None:
         """Diff two synchronized lists."""
         self.logger.debug("Differ::_diff_synced_lists:  Starting...")
         self.logger.debug(
@@ -255,8 +283,8 @@ class Differ:
                     parentref=ridx)
 
     def _diff_arrays_of_scalars(
-        self, path: YAMLPath, lhs: list, rhs: list, node_coord: NodeCoords,
-        **kwargs
+        self, path: YAMLPath, lhs: CommentedSeq, rhs: CommentedSeq,
+        node_coord: NodeCoords, **kwargs
     ) -> None:
         """Diff two lists of scalars."""
         self.logger.debug(
@@ -305,7 +333,8 @@ class Differ:
                         parentref=idx))
 
     def _diff_arrays_of_hashes(
-        self, path: YAMLPath, lhs: list, rhs: list, node_coord: NodeCoords
+        self, path: YAMLPath, lhs: CommentedSeq, rhs: CommentedSeq,
+        node_coord: NodeCoords
     ) -> None:
         """Diff two lists-of-dictionaries."""
         self.logger.debug(
@@ -344,15 +373,7 @@ class Differ:
 
         # Perform either a KEY or DEEP comparison; either way, the elements
         # must first be synchronized based on their identity key values.
-        id_key: str = ""
-        if len(rhs) > 0 and isinstance(rhs[0], dict):
-            id_key = self.config.aoh_diff_key(
-                NodeCoords(rhs[0], rhs, 0), rhs[0])
-            self.logger.debug(
-                "Differ::_diff_arrays_of_hashes:  RHS AoH yielded id_key:"
-                "  {}.".format(id_key))
-
-        syn_pairs = Differ.synchronize_lods_by_key(lhs, rhs, id_key)
+        syn_pairs = self.synchronize_lods_by_key(path, lhs, rhs)
         self.logger.debug(
             "Got synchronized pairs of Array elements at YAML Path, {}:"
             .format(path if path else "/"),
@@ -395,7 +416,7 @@ class Differ:
                         parentref=lidx))
 
     def _diff_lists(
-        self, path: YAMLPath, lhs: list, rhs: list, **kwargs
+        self, path: YAMLPath, lhs: CommentedSeq, rhs: CommentedSeq, **kwargs
     ) -> None:
         """Diff two lists."""
         self.logger.debug(
@@ -411,7 +432,7 @@ class Differ:
         parentref: Any = kwargs.pop("parentref", None)
         node_coord = NodeCoords(rhs, parent, parentref)
         if len(rhs) > 0:
-            if isinstance(rhs[0], dict):
+            if isinstance(rhs[0], CommentedMap):
                 # This list is an Array-of-Hashes
                 self._diff_arrays_of_hashes(path, lhs, rhs, node_coord)
             else:
@@ -432,11 +453,11 @@ class Differ:
             data=rhs)
 
         # If the roots are different, delete all LHS and add all RHS.
-        lhs_is_dict = isinstance(lhs, dict)
-        lhs_is_list = isinstance(lhs, list)
+        lhs_is_dict = isinstance(lhs, CommentedMap)
+        lhs_is_list = isinstance(lhs, CommentedSeq)
         lhs_is_scalar = not (lhs_is_dict or lhs_is_list)
-        rhs_is_dict = isinstance(rhs, dict)
-        rhs_is_list = isinstance(rhs, list)
+        rhs_is_dict = isinstance(rhs, CommentedMap)
+        rhs_is_list = isinstance(rhs, CommentedSeq)
         rhs_is_scalar = not (rhs_is_dict or rhs_is_list)
         same_types = (
             (lhs_is_dict and rhs_is_dict)
@@ -456,7 +477,7 @@ class Differ:
 
     @classmethod
     def synchronize_lists_by_value(
-        cls, lhs: list, rhs: list
+        cls, lhs: CommentedSeq, rhs: CommentedSeq
     ) -> List[Tuple[
         Optional[int], Optional[Any], Optional[int], Optional[Any]
     ]]:
@@ -490,16 +511,23 @@ class Differ:
 
         return syn_pairs
 
-    @classmethod
     #pylint: disable=too-many-locals
     def synchronize_lods_by_key(
-        cls, lhs: list, rhs: list, key_attr: str
+        self, path: YAMLPath, lhs: CommentedSeq, rhs: CommentedSeq
     ) -> List[Tuple[
         Optional[int], Optional[Any], Optional[int], Optional[Any]
     ]]:
         """Synchronize two lists-of-dictionaries by identity key."""
         # Build a parallel index array to track the original RHS element
         # indexes of any surviving elements.
+        key_attr: str = ""
+        if len(rhs) > 0 and isinstance(rhs[0], CommentedMap):
+            (key_attr, _) = self.config.aoh_diff_key(
+                NodeCoords(rhs[0], rhs, 0))
+            self.logger.debug(
+                "Differ::synchronize_lods_by_key:  RHS AoH yielded key_attr:"
+                "  {}.".format(key_attr))
+
         rhs_reduced = []
         for original_idx, val in enumerate(rhs):
             rhs_reduced.append((original_idx, val))
@@ -510,17 +538,42 @@ class Differ:
         for lhs_idx, lhs_ele in enumerate(lhs):
             if not key_attr in lhs_ele:
                 # Impossible to match this LHS record to any RHS record
+                self.logger.debug(
+                    "LHS record has no identity key, {}, for record at {}:"
+                    .format(key_attr, path),
+                    data=lhs_ele,
+                    prefix="Differ::synchronize_lods_by_key:  ")
                 syn_pairs.append((lhs_idx, lhs_ele, None, None))
                 continue
 
             del_index = -1
             for reduced_idx, rhs_pair in enumerate(rhs_reduced):
-                (_, rhs_ele) = rhs_pair
-                if not key_attr in rhs_ele:
+                (rhs_idx, rhs_ele) = rhs_pair
+
+                # Check for a custom identity key assignment for this record
+                use_key = key_attr
+                (alt_key, is_user_key) = self.config.aoh_diff_key(
+                    NodeCoords(rhs_ele, rhs, rhs_idx))
+
+                if is_user_key and alt_key:
+                    use_key = alt_key
+                    self.logger.debug(
+                        "Using alternate key, {}, for record at {}[{}]."
+                        .format(use_key, path, rhs_idx),
+                        data=rhs_ele,
+                        prefix="Differ::synchronize_lods_by_key:  ")
+                else:
+                    self.logger.debug(
+                        "Using inferred key, {}, for record at {}[{}]."
+                        .format(use_key, path, rhs_idx),
+                        data=rhs_ele,
+                        prefix="Differ::synchronize_lods_by_key:  ")
+
+                if not use_key in rhs_ele:
                     # Impossible to match this RHS record to any LHS record
                     continue
 
-                if rhs_ele[key_attr] == lhs_ele[key_attr]:
+                if rhs_ele[use_key] == lhs_ele[use_key]:
                     del_index = reduced_idx
                     break
 
@@ -537,10 +590,10 @@ class Differ:
         return syn_pairs
 
     @classmethod
-    def _get_key_indicies(cls, data: dict) -> Dict[Any, int]:
+    def _get_key_indicies(cls, data: CommentedMap) -> Dict[Any, int]:
         """Get a dictionary mapping of keys to relative positions."""
         key_map = {}
-        if isinstance(data, dict):
+        if isinstance(data, CommentedMap):
             for idx, key in enumerate(data.keys()):
                 key_map[key] = idx
         return key_map
